#!/bin/sh

# backlightj v1.04 (11th march 2011) by Andy Davison
#  Simple automatic backlight control with fading for the Joggler.


## First off, check what system we're running on and if we can access the backlight.
if [ -e /sys/class/backlight/openframe-bl/brightness ]; then
	BLIF="/sys/class/backlight/openframe-bl/brightness"
	NATIVE="0"
	SLEEP="sleep 0.001"
	if [ ! -w /sys/class/backlight/openframe-bl/brightness ]; then
		echo "Permissions do not currently allow you to set the backlight level."
		echo "Make sure that the udev rule is in place in /etc/udev/rules.d/"
	fi
elif [ -e /proc/blctrl ]; then
	BLIF="/proc/blctrl"
	NATIVE="1"
	SLEEP="usleep 3000"
else
	echo "No known backlight interface found."
	exit 0
fi

## Stop if we're fed a backlight value we can't handle, or set the requested brightness.
if [ "$1" -gt "100" 2>/dev/null ]; then
	echo "Usage: $0 [0-100]"
	exit 0
else
	SETBL="$1"
fi

## Then grab the current backlight value, adjusting if this is not the native OS.
CRNTBL=`cat $BLIF`
if [ "$NATIVE" -lt "1" ]; then
	CRNTBL=$(($CRNTBL*3))
fi

if [ ! "$1" ]; then
	
	echo $CRNTBL

else
	
	## If we're in automatic mode, run a monitoring loop.
	if [ "$1" = "auto" ] || [ "$1" -eq "999" ]; then
		
		LASTBL="$CRNTBL"
		
		while true
		do

			TIME=`date +%H`
			LIGHT=`readlightsensor`
			PREFBL=$(($LIGHT))			
			DELTABL=$(($LASTBL-$PREFBL))
			
			# Here we deal with 'night and day' variations.
			if [ "$TIME" -ge 22 ] || [ "$TIME" -lt 6 ]; then
				MODE="Night"
				# Night hysteresis values, so that less change is required to dim display, more is required to light it.
				HYSUP="-4"
				HYSDOWN="2"
			else
				MODE="Day"
				# Normal hysteresis values, so we don't 'bounce' the backlight with minor light changes.
				HYSUP="-2"
				HYSDOWN="3"
			fi
			
			if [ "$2" = "debug" 2>/dev/null ]; then
				echo "Current Mode is:       $MODE"
				echo "Light sensor reads:    $LIGHT"
				echo "Backlight % should be: $PREFBL"
				echo "Previous backlight % : $LASTBL"
				echo "Backlight delta %:     $DELTABL"
				echo
			fi
			
			if [ "$DELTABL" -le $HYSUP ] || [ "$DELTABL" -gt $HYSDOWN ]; then
				if [ "$2" == "debug" 2>/dev/null ]; then
					echo "Issuing: $0 $PREFBL"
					echo
				fi
				$0 $PREFBL
			fi
						
			LASTBL="$PREFBL"
			sleep 2

		done
		
	fi
	
	## From here we deal with an issued backlight value.
	if [ "$2" = "debug" ]; then
		echo "Current backlight level: $CRNTBL"
		echo "Set backlight level to:  $SETBL"
	fi

	if [ "$SETBL" -gt "$CRNTBL" ]; then
		STEP="1"
	else
		STEP="-1"
	fi

	for b in `seq $CRNTBL $STEP $SETBL`;
	do

		if [ "$NATIVE" -lt "1" ]; then
			b=$(($b/3))
		fi
		
		[ "$2" != "debug" ] || echo $b

		echo "$b" > $BLIF
		$SLEEP

	done

fi
